﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using FarPoint.Win.Spread;
using FarPoint.Win.Spread.Model;
using PowerOffice.SmartClient.DocumentCenter.Views.PowerFormsDesignerView;
using PowerOffice.SmartClient.DocumentCenter.Dialogs.Controls;
using PowerOffice.SmartClient.DocumentCenter.Views.PowerFormsDesignerView.Actions;

namespace FpSpreadBugHunt1
{
    public partial class Form1 : Form
    {

        #region Never mind us, part of the main project

        private void ApplyBorder(params BorderEdgeDetail[] commands)
        {
            this.spreadSheet.UndoManager.PerformUndoAction(new CellSetBorderAction(this.spreadSheet.ActiveSheet, commands));
        }

        private void ApplyGenericBorderStyle(GenericBorderStyle borderStyle)
        {
            switch (borderStyle)
            {
                case GenericBorderStyle.None:
                    ApplyBorder(
                        new BorderEdgeDetail(BorderEdges.Left, LineStyle.None, Color.Black),
                        new BorderEdgeDetail(BorderEdges.Top, LineStyle.None, Color.Black),
                        new BorderEdgeDetail(BorderEdges.Right, LineStyle.None, Color.Black),
                        new BorderEdgeDetail(BorderEdges.Bottom, LineStyle.None, Color.Black),
                        new BorderEdgeDetail(BorderEdges.Middle, LineStyle.None, Color.Black),
                        new BorderEdgeDetail(BorderEdges.Center, LineStyle.None, Color.Black));
                    break;

                case GenericBorderStyle.Top:
                    ApplyBorder(
                        new BorderEdgeDetail(BorderEdges.Top, LineStyle.ThinLine, Color.Black));
                    break;

                case GenericBorderStyle.Bottom:
                    ApplyBorder(
                        new BorderEdgeDetail(BorderEdges.Bottom, LineStyle.ThinLine, Color.Black));
                    break;

                case GenericBorderStyle.Left:
                    ApplyBorder(
                        new BorderEdgeDetail(BorderEdges.Left, LineStyle.ThinLine, Color.Black));
                    break;

                case GenericBorderStyle.Right:
                    ApplyBorder(
                        new BorderEdgeDetail(BorderEdges.Right, LineStyle.ThinLine, Color.Black));
                    break;

                case GenericBorderStyle.All:
                    ApplyBorder(
                        new BorderEdgeDetail(BorderEdges.Left, LineStyle.ThinLine, Color.Black),
                        new BorderEdgeDetail(BorderEdges.Top, LineStyle.ThinLine, Color.Black),
                        new BorderEdgeDetail(BorderEdges.Right, LineStyle.ThinLine, Color.Black),
                        new BorderEdgeDetail(BorderEdges.Bottom, LineStyle.ThinLine, Color.Black),
                        new BorderEdgeDetail(BorderEdges.Middle, LineStyle.ThinLine, Color.Black),
                        new BorderEdgeDetail(BorderEdges.Center, LineStyle.ThinLine, Color.Black));
                    break;

                case GenericBorderStyle.Outer:
                    ApplyBorder(
                        new BorderEdgeDetail(BorderEdges.Left, LineStyle.ThinLine, Color.Black),
                        new BorderEdgeDetail(BorderEdges.Top, LineStyle.ThinLine, Color.Black),
                        new BorderEdgeDetail(BorderEdges.Right, LineStyle.ThinLine, Color.Black),
                        new BorderEdgeDetail(BorderEdges.Bottom, LineStyle.ThinLine, Color.Black));
                    break;

                case GenericBorderStyle.ThickOuter:
                    ApplyBorder(
                        new BorderEdgeDetail(BorderEdges.Left, LineStyle.ThickLine, Color.Black),
                        new BorderEdgeDetail(BorderEdges.Top, LineStyle.ThickLine, Color.Black),
                        new BorderEdgeDetail(BorderEdges.Right, LineStyle.ThickLine, Color.Black),
                        new BorderEdgeDetail(BorderEdges.Bottom, LineStyle.ThickLine, Color.Black));
                    break;

                case GenericBorderStyle.DoubleBottom:
                    ApplyBorder(
                        new BorderEdgeDetail(BorderEdges.Bottom, LineStyle.DoubleLine, Color.Black));
                    break;

                case GenericBorderStyle.ThickBottom:
                    ApplyBorder(
                        new BorderEdgeDetail(BorderEdges.Bottom, LineStyle.ThickLine, Color.Black));
                    break;

                case GenericBorderStyle.TopAndBottom:
                    ApplyBorder(
                        new BorderEdgeDetail(BorderEdges.Bottom, LineStyle.ThinLine, Color.Black),
                        new BorderEdgeDetail(BorderEdges.Bottom, LineStyle.ThinLine, Color.Black));
                    break;

                case GenericBorderStyle.TopAndDoubleBottom:
                    ApplyBorder(
                        new BorderEdgeDetail(BorderEdges.Bottom, LineStyle.ThinLine, Color.Black),
                        new BorderEdgeDetail(BorderEdges.Bottom, LineStyle.DoubleLine, Color.Black));
                    break;

                case GenericBorderStyle.TopAndThickBottom:
                    ApplyBorder(
                        new BorderEdgeDetail(BorderEdges.Bottom, LineStyle.ThinLine, Color.Black),
                        new BorderEdgeDetail(BorderEdges.Bottom, LineStyle.ThickLine, Color.Black));
                    break;
            }
        }
        
#endregion

        #region Set up and print spread

        /// <summary>
        /// Prints an entire spread
        /// </summary>
        /// <param name="printDialog"></param>
        private void PrintForm(PrintDialog printDialog)
        {
            int firstPage, lastPage;
            switch (printDialog.PrinterSettings.PrintRange)
            {
                case System.Drawing.Printing.PrintRange.CurrentPage:
                    firstPage = lastPage = this.spreadSheet.ActiveSheetIndex;
                    break;

                case System.Drawing.Printing.PrintRange.SomePages:
                    firstPage = printDialog.PrinterSettings.FromPage - 1;
                    lastPage = printDialog.PrinterSettings.ToPage - 1;
                    break;

                case System.Drawing.Printing.PrintRange.AllPages:
                default:
                    firstPage = 0;
                    lastPage = this.spreadSheet.Sheets.Count - 1;
                    break;
            }

            if (printDialog.PrinterSettings.Collate)
            {
                for (int copy = 0; copy < printDialog.PrinterSettings.Copies; copy++)
                {
                    for (int page = firstPage; page <= lastPage; page++)
                    {
                        SheetView sheet = this.spreadSheet.Sheets[page];
                        PrintPage(printDialog, sheet);
                    }
                }
            }
            else
            {
                for (int page = firstPage; page <= lastPage; page++)
                {
                    for (int copy = 0; copy < printDialog.PrinterSettings.Copies; copy++)
                    {
                        SheetView sheet = this.spreadSheet.Sheets[page];
                        PrintPage(printDialog, sheet);
                    }
                }
            }
        }

        /// <summary>
        /// Prints a single sheet
        /// </summary>
        /// <param name="printDialog"></param>
        /// <param name="sheet"></param>
        private void PrintPage(PrintDialog printDialog, SheetView sheet)
        {
            sheet.PrintInfo = ToPrintInfo(sheet, printDialog.PrinterSettings);
            GridLine oldHorizontalGridLine = sheet.HorizontalGridLine;
            GridLine oldVerticalGridLine = sheet.VerticalGridLine;
            try
            {
                sheet.HorizontalGridLine = new GridLine(GridLineType.Flat, Color.Empty);
                sheet.VerticalGridLine = new GridLine(GridLineType.Flat, Color.Empty);
                this.spreadSheet.PrintSheet(sheet);
            }
            finally
            {
                sheet.HorizontalGridLine = oldHorizontalGridLine;
                sheet.VerticalGridLine = oldVerticalGridLine;
            }
        }

        /// <summary>
        /// This function converts printer settings from a print dialog into a PrintInfo structure.
        /// </summary>
        /// <param name="sheet"></param>
        /// <param name="printerSettings"></param>
        /// <returns></returns>
        private PrintInfo ToPrintInfo(SheetView sheet, System.Drawing.Printing.PrinterSettings printerSettings)
        {
            PrintInfo printInfo = new PrintInfo(sheet.PrintInfo);
            printInfo.FirstPageNumber = 1;
            printInfo.JobName = "PowerOffice PowerForm Print";
            printInfo.Margin = new PrintMargin(printerSettings.DefaultPageSettings.Margins.Left,
                printerSettings.DefaultPageSettings.Margins.Top,
                printerSettings.DefaultPageSettings.Margins.Right,
                printerSettings.DefaultPageSettings.Margins.Bottom,
                0,
                0);
            printInfo.Orientation = printerSettings.DefaultPageSettings.Landscape ? PrintOrientation.Landscape : PrintOrientation.Portrait;
            printInfo.PageOrder = PrintPageOrder.DownThenOver;
            printInfo.PaperSize = printerSettings.DefaultPageSettings.PaperSize;
            printInfo.PaperSource = printerSettings.DefaultPageSettings.PaperSource;
            printInfo.Printer = printerSettings.PrinterName;
            printInfo.PrintNotes = PrintNotes.AtEnd;
            printInfo.PrintShapes = true;
            printInfo.ShowBorder = false;
            printInfo.ShowColumnHeader = PrintHeader.Hide;
            printInfo.ShowRowHeader = PrintHeader.Hide;
            printInfo.ShowGrid = false;
            printInfo.PrintType = PrintType.CurrentPage;
            printInfo.ShowColor = printerSettings.DefaultPageSettings.Color;
            printInfo.ShowPrintDialog = false;
            printInfo.UseSmartPrint = false;
            return printInfo;
        }

        #endregion

        /// <summary>
        /// This function (or rather a very similar one) creates all the sheets in the FpSpread.
        /// </summary>
        /// <param name="name"></param>
        /// <param name="rows"></param>
        /// <param name="columns"></param>
        /// <returns></returns>
        private SheetView CreateSheet(string name, int rows, int columns)
        {
            SheetView newSheet = new SheetView();
            newSheet.DefaultStyle.Font = this.spreadSheet.Font;
            newSheet.ColumnCount = columns;
            newSheet.RowCount = rows;
            newSheet.SheetName = name;
            newSheet.AllowNoteEdit = true;
            newSheet.DefaultStyle.Locked = true;
            newSheet.Protect = false;
            newSheet.DisplayZero = true;
            newSheet.SelectionStyle = SelectionStyles.SelectionRenderer;
            return newSheet;
        }

        public Form1()
        {
            InitializeComponent();
        }

        private void toolStripButton1_Click(object sender, EventArgs e)
        {
            SheetView sheet = CreateSheet("MyTestSheet", 50, 25);
            this.spreadSheet.Sheets.Add(sheet);
            this.spreadSheet.ActiveSheet = sheet;

            this.spreadSheet.ActiveSheet.AddSelection(5, 3, 5, 5);
            ApplyGenericBorderStyle(GenericBorderStyle.ThickOuter);
        }

        private void toolStripButton2_Click(object sender, EventArgs e)
        {
            this.printDialog.AllowSomePages = true;
            this.printDialog.AllowCurrentPage = true;
            this.printDialog.AllowPrintToFile = false;
            this.printDialog.AllowSelection = false;
            this.printDialog.PrinterSettings.MaximumPage = this.spreadSheet.Sheets.Count;
            this.printDialog.PrinterSettings.MinimumPage = 1;
            this.printDialog.PrinterSettings.FromPage = this.printDialog.PrinterSettings.MinimumPage;
            this.printDialog.PrinterSettings.ToPage = this.printDialog.PrinterSettings.MaximumPage;
            if (this.printDialog.ShowDialog(this.TopLevelControl) == DialogResult.OK)
            {
                PrintForm(this.printDialog);
            }
        }
    }
}
