﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;
using FarPoint.Win;

namespace PowerOffice.SmartClient.DocumentCenter.Dialogs.Controls
{
    class BorderEdgeDetail
    {
        private Color color = Color.Black;
        private LineStyle style = LineStyle.None;

        public BorderEdgeDetail(BorderEdges edge, LineStyle lineStyle, Color color)
        {
            this.Edge = edge;
            this.style = lineStyle;
            this.color = color;
        }

        public event EventHandler ColorChanged;
        public event EventHandler StyleChanged;

        public BorderEdges Edge { get; private set; }
        
        public float Width
        {
            get
            {
                switch (this.style)
                {
                    case LineStyle.Dotted:
                        return 1;

                    case LineStyle.DashDotDot:
                        return 1;

                    case LineStyle.DashDot:
                        return 1;

                    case LineStyle.Dashed:
                        return 1;

                    case LineStyle.MediumDashDotDot:
                        return 2;

                    case LineStyle.MediumDashDot:
                        return 2;

                    case LineStyle.MediumDashed:
                        return 2;

                    case LineStyle.MediumLine:
                        return 2;

                    case LineStyle.ThickLine:
                        return 3;

                    case LineStyle.DoubleLine:
                        return 3;

                    case LineStyle.ThinLine:
                    default:
                        return 1;
                }
            }
        }

        public LineStyle Style
        {
            get
            {
                return this.style;
            }
            set
            {
                this.style = value;
            }
        }

        public Color Color
        {
            get
            {
                return this.color;
            }
            set
            {
                this.color = value;
            }
        }

        public bool Visible
        {
            get
            {
                return this.Style != LineStyle.None;
            }
        }

        protected virtual void OnColorChanged(EventArgs eventArgs)
        {
            if (this.ColorChanged != null)
                ColorChanged(this, eventArgs);
        }

        private void OnStyleChanged(EventArgs eventArgs)
        {
            if (this.StyleChanged != null)
                StyleChanged(this, eventArgs);
        }

        public BorderEdgeDetail Clone()
        {
            return new BorderEdgeDetail(this.Edge, this.Style, this.Color);
        }
    }
}
