using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using FarPoint.Win.Spread;
using FarPoint.Win.Spread.Model;

/*
	Creator: morten
	Date:    2/18/2008 1:30:19 PM
	
	*** Version control: ***
	Revision of last commit:     $Revision: 556 $
	Last changed date:           $LastChangedDate: 2008-03-15 12:57:13 +0100 (lø, 15 mar 2008) $
	Last changed by author:      $Author: mal $
	Repository URL of this file: $URL: svn://svn.poweroffice.com/PowerOffice/trunk/source/Module/DocumentCenter/DocumentCenter/Views/PowerFormsDesignerView/CellSelection.cs $
	
	Copyright (c) 2007 PowerOffice AS, All rights reserved
*/

namespace PowerOffice.SmartClient.DocumentCenter.Views.PowerFormsDesignerView
{
    /// <summary>
    /// Summary description for CellSelection.
    ///
    /// </summary>
    class CellSelection
    {
        public CellSelection(SheetView sheet)
        {
            this.Sheet = sheet;
            this.ActiveRow = sheet.ActiveRow.Index;
            this.ActiveColumn = sheet.ActiveColumn.Index;
            if (sheet.SelectionCount > 0)
            {
                CellRange range = sheet.GetSelection(0);
                this.Row = (range.Row >= 0) ? range.Row : 0;
                this.Column = (range.Column >= 0) ? range.Column : 0;
                this.RowCount = (range.RowCount >= 0) ? range.RowCount : sheet.RowCount;
                this.ColumnCount = (range.ColumnCount >= 0) ? range.ColumnCount : sheet.ColumnCount;
            }
            else
            {
                this.Row = this.ActiveRow;
                this.Column = this.ActiveColumn;
                this.RowCount = 1;
                this.ColumnCount = 1;
            }

        }

        public CellSelection(int activeRow, int activeColumn, int firstRow, int firstColumn, int rowCount, int columnCount)
        {
            this.ActiveRow = activeRow;
            this.ActiveColumn = activeColumn;
            this.Row = firstRow;
            this.Column = firstColumn;
            this.RowCount = rowCount;
            this.ColumnCount = columnCount;
        }

        public SheetView Sheet { get; private set; }
        public int ActiveRow { get; private set; }
        public int ActiveColumn { get; private set; }
        public int Row { get; private set; }
        public int RowCount { get; private set; }
        public int Column { get; private set; }
        public int ColumnCount { get; private set; }

        public IEnumerable<Cell> Cells
        {
            get
            {
                for (int row = this.Row; row < this.Row + this.RowCount; row++)
                {
                    for (int column = this.Column; column < this.Column + this.ColumnCount; column++)
                        yield return this.Sheet.Cells[row, column];
                }
            }
        }

        public IEnumerable<Cell> LeftCells
        {
            get
            {
                for (int row = this.Row; row < this.Row + this.RowCount; row++)
                    yield return this.Sheet.Cells[row, this.Column];
            }
        }

        public IEnumerable<Cell> TopCells
        {
            get
            {
                for (int column = this.Column; column < this.Column + this.ColumnCount; column++)
                    yield return this.Sheet.Cells[this.Row, column];
            }
        }

        public IEnumerable<Cell> RightCells
        {
            get
            {
                for (int row = this.Row; row < this.Row + this.RowCount; row++)
                    yield return this.Sheet.Cells[row, this.Column + this.ColumnCount - 1];
            }
        }

        public IEnumerable<Cell> BottomCells
        {
            get
            {
                for (int column = this.Column; column < this.Column + this.ColumnCount; column++)
                    yield return this.Sheet.Cells[this.Row + this.RowCount - 1, column];
            }
        }

        public IEnumerable<Cell> InnerCells
        {
            get
            {
                for (int row = this.Row + 1; row < this.Row + this.RowCount - 1; row++)
                {
                    for (int column = this.Column + 1; column < this.Column + this.ColumnCount - 1; column++)
                        yield return this.Sheet.Cells[row, column];
                }
            }
        }
    }
}
