using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using FarPoint.Win;
using FarPoint.Win.Spread;
using FarPoint.Win.Spread.Model;
using System.Drawing;
using PowerOffice.SmartClient.DocumentCenter.Dialogs.Controls;

/*
	Creator: morten
	Date:    2/12/2008 11:30:49 AM
	
	*** Version control: ***
	Revision of last commit:     $Revision: 626 $
	Last changed date:           $LastChangedDate: 2008-03-26 10:35:20 +0100 (on, 26 mar 2008) $
	Last changed by author:      $Author: mal $
	Repository URL of this file: $URL: svn://svn.poweroffice.com/PowerOffice/trunk/source/Module/DocumentCenter/DocumentCenter/Views/PowerFormsDesignerView/Actions/CellSetBorderAction.cs $
	
	Copyright (c) 2007 PowerOffice AS, All rights reserved
*/

namespace PowerOffice.SmartClient.DocumentCenter.Views.PowerFormsDesignerView.Actions
{
    /// <summary>
    /// Setter ramme p et utvalg av celler.
    /// </summary>
    class CellSetBorderAction: CellSelectionAction<IBorder>
    {
        private IEnumerable<BorderEdgeDetail> commands;

        public CellSetBorderAction(SheetView sheet, params BorderEdgeDetail[] borderCommands)
            : base(sheet)
        {
            this.commands = borderCommands;
        }

        protected override IBorder GetOriginalState(Cell cell)
        {
            return cell.Border;
        }

        protected sealed override void SetNewState(Cell cell)
        {
            // Denne funksjonen kalles aldri siden PerformUndoAction er overstyrt.
            throw new NotImplementedException();
        }

        protected override void ResetToOriginalState(Cell cell, IBorder originalState)
        {
            cell.Border = originalState;
        }

        public override bool PerformUndoAction(object sender)
        {
            foreach (BorderEdgeDetail command in this.commands)
            {
                switch (command.Edge)
                {
                    case BorderEdges.Left:
                        ApplyLeftBorder((ComplexBorderSideStyle)command.Style, command.Color);
                        break;

                    case BorderEdges.Top:
                        ApplyTopBorder((ComplexBorderSideStyle)command.Style, command.Color);
                        break;

                    case BorderEdges.Right:
                        ApplyRightBorder((ComplexBorderSideStyle)command.Style, command.Color);
                        break;

                    case BorderEdges.Bottom:
                        ApplyBottomBorder((ComplexBorderSideStyle)command.Style, command.Color);
                        break;

                    case BorderEdges.Middle:
                        ApplyMiddleBorder((ComplexBorderSideStyle)command.Style, command.Color);
                        break;

                    case BorderEdges.Center:
                        ApplyCenterBorder((ComplexBorderSideStyle)command.Style, command.Color);
                        break;
                }
            }
            return true;
        } 

        private void ApplyRightBorder(ComplexBorderSideStyle borderStyle, Color color)
        {
            int start = this.Selection.Row;
            int end = start + this.Selection.RowCount;
            int col = this.Selection.Column + this.Selection.ColumnCount - 1;
            for (int row = start; row < end; row++)
            {
                Cell cell = this.Sheet.Cells[row, col];
                CellBorder border = new CellBorder(cell);
                border.Right = CreateBorderSide(borderStyle, color);
                cell.Border = border.ToComplexBorder();
            }
        }

        private void ApplyLeftBorder(ComplexBorderSideStyle borderStyle, Color color)
        {
            int start = this.Selection.Row;
            int end = start + this.Selection.RowCount;
            int col = this.Selection.Column;
            for (int row = start; row < end; row++)
            {
                Cell cell = this.Sheet.Cells[row, col];
                CellBorder border = new CellBorder(cell);
                border.Left = CreateBorderSide(borderStyle, color);
                cell.Border = border.ToComplexBorder();
            }
        }

        private void ApplyTopBorder(ComplexBorderSideStyle borderStyle, Color color)
        {
            int start = this.Selection.Column;
            int end = start + this.Selection.ColumnCount;
            int row = this.Selection.Row;
            for (int col = start; col < end; col++)
            {
                Cell cell = this.Sheet.Cells[row, col];
                CellBorder border = new CellBorder(cell);
                border.Top = CreateBorderSide(borderStyle, color);
                cell.Border = border.ToComplexBorder();
            }
        }

        private void ApplyBottomBorder(ComplexBorderSideStyle borderStyle, Color color)
        {
            int start = this.Selection.Column;
            int end = start + this.Selection.ColumnCount;
            int row = this.Selection.Row + this.Selection.RowCount - 1;
            for (int col = start; col < end; col++)
            {
                Cell cell = this.Sheet.Cells[row, col];
                CellBorder border = new CellBorder(cell);
                border.Bottom = CreateBorderSide(borderStyle, color);
                cell.Border = border.ToComplexBorder();
            }
        }

        private void ApplyCenterBorder(ComplexBorderSideStyle borderStyle, Color color)
        {
            int startCol = this.Selection.Column;
            int endCol = startCol + this.Selection.ColumnCount - 1;
            int startRow = this.Selection.Row;
            int endRow = startRow + this.Selection.RowCount;
            for (int row = startRow; row < endRow; row++)
            {
                for (int col = startCol; col <= endCol; col++)
                {
                    Cell cell = this.Sheet.Cells[row, col];
                    CellBorder border = new CellBorder(cell);
                    if (col > startCol)
                        border.Left = CreateBorderSide(borderStyle, color);
                    if (col < endCol)
                        border.Right = CreateBorderSide(borderStyle, color);
                    cell.Border = border.ToComplexBorder();
                }
            }
        }

        private void ApplyMiddleBorder(ComplexBorderSideStyle borderStyle, Color color)
        {
            int startCol = this.Selection.Column;
            int endCol = startCol + this.Selection.ColumnCount;
            int startRow = this.Selection.Row;
            int endRow = startRow + this.Selection.RowCount - 1;
            for (int row = startRow; row <= endRow; row++)
            {
                for (int col = startCol; col < endCol; col++)
                {
                    Cell cell = this.Sheet.Cells[row, col];
                    CellBorder border = new CellBorder(cell);
                    if (row > startRow)
                        border.Top = CreateBorderSide(borderStyle, color);
                    if (row < endRow)
                        border.Bottom = CreateBorderSide(borderStyle, color);
                    cell.Border = border.ToComplexBorder();
                }
            }
        }

        private ComplexBorderSide CreateBorderSide(ComplexBorderSideStyle borderStyle, Color color)
        {
            return borderStyle != ComplexBorderSideStyle.None ? new ComplexBorderSide(borderStyle, color) : null;
        }

        private class CellBorder
        {
            public CellBorder(Cell cell)
            {
                if (cell.Border is ComplexBorder)
                {
                    ComplexBorder border = (ComplexBorder)cell.Border;
                    this.Left = border.Left;
                    this.Top = border.Top;
                    this.Right = border.Right;
                    this.Bottom = border.Bottom;
                }
            }

            public ComplexBorderSide Left { get; set; }
            public ComplexBorderSide Top { get; set; }
            public ComplexBorderSide Right { get; set; }
            public ComplexBorderSide Bottom { get; set; }

            public ComplexBorder ToComplexBorder()
            {
                return new ComplexBorder(this.Left, this.Top, this.Right, this.Bottom);
            }
        }
    }
}
